import 'package:dio/dio.dart';
import 'dart:async';
import 'package:connectivity/connectivity.dart';
import 'package:flutter/material.dart';
import 'CustomDialog.dart';
import 'dialog.dart';

///网络请求工具类
/// 牛孝捡
/// 20200508
/* * 封装 restful 请求 * * GET、POST、DELETE、PATCH * 主要作用为统一处理相关事务： * - 统一处理请求前缀； * - 统一打印请求信息； * - 统一打印响应信息； * - 统一打印报错信息； */
class HttpUtils {
  /// global dio object
  static Dio dio;

  /// default options
  // ignore: non_constant_identifier_names
  static String API_PREFIX = 'http://localhost:8080/';
  // ignore: non_constant_identifier_names
  static int CONNECT_TIMEOUT = 10000;
  // ignore: non_constant_identifier_names
  static int RECEIVE_TIMEOUT = 3000;

  /// http request methods
  static const String GET = 'get';
  static const String POST = 'post';
  static const String PUT = 'put';
  static const String PATCH = 'patch';
  static const String DELETE = 'delete';

  /// request method
  static Future<Map> request(BuildContext context, String url,
      {data, method, api, errorcallback}) async {
    Connectivity connectivity = Connectivity(); //监听网络状态
    ConnectivityResult previousResult = await connectivity.checkConnectivity();
    // ignore: unrelated_type_equality_checks
    if ("ConnectivityResult.none" == previousResult.toString()) {
      //无网络提示
      showDialog(
          context: context,
          barrierDismissible: false,
          builder: (_) {
            return CustomDialog(
              title: '提示',
              content: '暂无网络，请连接网络',
              confirmTextColor: Colors.blue,
              confirmContent: '确定',
              isCancel: false,
              confirmCallback: () {},
            );
          });
    } else {
      ProgressDialog.showProgress(context,
          child: new Column(
            mainAxisAlignment: MainAxisAlignment.center,
            crossAxisAlignment: CrossAxisAlignment.center,
            children: <Widget>[
              new CircularProgressIndicator(),
              new Padding(
                padding: const EdgeInsets.only(
                  top: 20.0,
                ),
                child: new Text(
                  "loading...",
                  style: new TextStyle(
                    fontSize: 16.0,
                    fontWeight: FontWeight.w500,
                    color: Colors.black26,
                  ),
                ),
              ),
            ],
          ));

      data = data ?? {};
      method = method ?? 'GET';

      /// restful 请求处理
      /// /gysw/search/hist/:user_id user_id=27
      /// 最终生成 url 为 /gysw/search/hist/27
      data.forEach((key, value) {
        if (url.indexOf(key) != -1) {
          url = url.replaceAll(':$key', value.toString());
        }
      });

      /// 打印请求相关信息：请求地址、请求方式、请求参数
//    print('请求地址：【' + method + '  ' + url + '】');
//    print('请求参数：' + data.toString());

      Dio dio = api == null ? createInstance() : createInstance(api: api);
      var result;

      try {
        Response response = await dio.request(url,
            data: data, options: new Options(method: method));

        result = response.data;
        ProgressDialog.dismiss(context);
        if (result['code'] == "500") {
          showDialog(
              context: context,
              barrierDismissible: false,
              builder: (_) {
                return CustomDialog(
                  title: '提示',
                  content: '系统错误请联系管理员',
                  confirmTextColor: Colors.blue,
                  confirmContent: '确定',
                  isCancel: false,
                  confirmCallback: () {},
                );
              });
        }

        /// 打印响应相关信息
        // print('响应数据：' + response.toString());
      } on DioError catch (e) {
        ProgressDialog.dismiss(context);
        if (errorcallback == null) {
          showDialog(
              context: context,
              barrierDismissible: false,
              builder: (_) {
                return CustomDialog(
                  title: '提示',
                  content: '系统错误请联系管理员',
                  confirmTextColor: Colors.blue,
                  confirmContent: '确定',
                  isCancel: false,
                  confirmCallback: () {},
                );
              });
        } else {
          errorcallback(); //执行自定义的方法
        }

        /// 打印请求失败相关信息
        print('请求出错：' + e.toString());
      } finally {
        clear(); // 清空 dio 对象
      }
      return result;
    }
  }

  /// 创建 dio 实例对象
  static Dio createInstance({api}) {
    if (dio == null) {
      /// 全局属性：请求前缀、连接超时时间、响应超时时间
      Options options = new Options(
        baseUrl: api == null ? API_PREFIX : api,
        //baseUrl: API_SYNC,
        connectTimeout: CONNECT_TIMEOUT,
        receiveTimeout: RECEIVE_TIMEOUT,
      );

      dio = new Dio(options);
    }

    return dio;
  }

  /// 清空 dio 对象
  static clear() {
    dio = null;
  }
}
